@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2002 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2002, 2006 Philip Blundell <philb@gnu.org>
+ *               2006, Florian Boor <florian@kernelconcepts.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +25,7 @@
 static unsigned long dbversion;
 static sqlite *sqliteh;
 
-static GSList *recurring_events, *one_shot_events;
+static GList *recurring_events, *one_shot_events;
 
 static const char *fname = "/.gpe/calendar";
 
@@ -46,17 +47,11 @@ GMemChunk *event_chunk, *recur_chunk;
 static gint
 event_sort_func (const event_t ev1, const event_t ev2)
 {
-  return (ev1->start > ev2->start) ? 1 : 0;
-}
-
-static gint
-event_sort_func_recur (const event_t ev1, const event_t ev2)
-{
+  if (!ev1->recur || !ev2->recur)
+    return ev1->start - ev2->start;
   recur_t r1 = ev1->recur;
   recur_t r2 = ev2->recur;
-  if(!r1->end) return (0);
-  else if (!r2->end) return (1);
-  else return (r1->end > r2->end) ? 0 : 1;
+  return r1->end - r2->end;
 }
 
 /* Add an event to the in-memory list */
@@ -66,24 +61,40 @@ event_db_add_internal (event_t ev)
   if (ev->uid >= uid)
     uid = ev->uid + 1;
 
-  if (ev->recur)
-    recurring_events = g_slist_insert_sorted (recurring_events, ev, 
-					      (GCompareFunc)event_sort_func_recur);
-  else
-    one_shot_events = g_slist_insert_sorted (one_shot_events, ev, 
-					     (GCompareFunc)event_sort_func);
-
+  ev->list = ev->recur ? &recurring_events : &one_shot_events;      
+  *ev->list = g_list_insert_sorted (*ev->list, ev, (GCompareFunc)event_sort_func); 
+      
   return TRUE;
 }
 
+/* Here we create an globally unique eventid, which we
+ * can use to reference this event in an vcal, etc. */
+static gchar *
+event_db_make_eventid (void)
+{
+  static char *hostname;
+  static char buffer [512];
+
+  if ((gethostname (buffer, sizeof (buffer) -1) == 0) &&
+     (buffer [0] != 0))
+    hostname = buffer;
+  else
+    hostname = "localhost";
+
+  return g_strdup_printf ("%lu.%lu@%s",
+                         (unsigned long) time (NULL),
+                         (unsigned long) getpid(),
+                         hostname); 
+}
+
 /* Remove an event from the in-memory list */
 static gboolean
 event_db_remove_internal (event_t ev)
 {
   if (ev->recur)
-    recurring_events = g_slist_remove (recurring_events, ev);
+    recurring_events = g_list_remove (recurring_events, ev);
   else
-    one_shot_events = g_slist_remove (one_shot_events, ev);
+    one_shot_events = g_list_remove (one_shot_events, ev);
 
   return TRUE;
 }
@@ -130,6 +141,10 @@ load_data_callback (void *arg, int argc, char **argv, char **names)
 	      ev->start += 12 * 60 * 60;
 	    }
 	}
+      else if (!strcasecmp (argv[0], "eventid"))
+	{
+	  ev->eventid = g_strdup (argv[1]);
+	}
       else if (!strcasecmp (argv[0], "rend"))
 	{
 	  recur_t r = event_db_get_recurrence (ev);
@@ -218,6 +233,14 @@ dbinfo_callback (void *arg, int argc, char **argv, char **names)
   return 0;
 }
 
+/**
+ * event_db_start:
+ *
+ * Initialise the event database for use. Needs to be called on application
+ * start before acessing events.
+ *
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
 gboolean
 event_db_start (void)
 {
@@ -283,24 +306,40 @@ load_details_callback (void *arg, int argc, char *argv[], char **names)
     {
       event_details_t evd = arg;
       if (!strcasecmp (argv[0], "summary") && !evd->summary)
-	evd->summary = g_strdup (argv[1]);
+        evd->summary = g_strdup (argv[1]);
       else if (!strcasecmp (argv[0], "description") && !evd->description)
-	evd->description = g_strdup (argv[1]);
+        evd->description = g_strdup (argv[1]);
+      else if (!strcasecmp (argv[0], "location") && !evd->location)
+        evd->location = g_strdup (argv[1]);
       else if (!strcasecmp (argv[0], "modified"))
-	{
-	  if (strchr (argv[1], '-'))
-	    parse_date (argv[1], &evd->modified, NULL);
-	  else
-	    evd->modified = strtoul (argv[1], NULL, 10);
-	}
+        {
+          if (strchr (argv[1], '-'))
+            parse_date (argv[1], &evd->modified, NULL);
+          else
+            evd->modified = strtoul (argv[1], NULL, 10);
+        }
       else if (!strcasecmp (argv[0], "sequence"))
-	evd->sequence = atoi (argv[1]);
+        evd->sequence = atoi (argv[1]);
       else if (!strcasecmp (argv[0], "category"))
-	evd->categories = g_slist_prepend (evd->categories, (gpointer)atoi (argv[1]));
+        evd->categories = g_slist_prepend (evd->categories, (gpointer)atoi (argv[1]));
     }
   return 0;
 }
 
+/**
+ * event_db_get_details:
+ * @ev: Event to get details for.
+ * 
+ * Retrieves detailed information for a given event and increments the 
+ * reference counter of the #event_details_t structure which will be 
+ * attached to the event. If the event details are already
+ * there they will not be loaded again but the reference counter is increased. 
+ * If you mant to make sure the event details are updated you need to use 
+ * event_db_forget_details() first. 
+ * 
+ *
+ * Returns: The #event_details_t event detail data attached to the event.
+ */
 event_details_t
 event_db_get_details (event_t ev)
 {
@@ -317,12 +356,12 @@ event_db_get_details (event_t ev)
 
       if (sqlite_exec_printf (sqliteh, "select tag,value from calendar where uid=%d",
 			      load_details_callback, evd, &err, ev->uid))
-	{
-	  gpe_error_box (err);
-	  free (err);
-	  g_free (evd);
-	  return NULL;
-	}
+        {
+          gpe_error_box (err);
+          free (err);
+          g_free (evd);
+          return NULL;
+        }
       
       ev->details = evd;
     }
@@ -332,43 +371,59 @@ event_db_get_details (event_t ev)
   return evd;
 }
 
+/**
+ * event_db_forget_details:
+ * @ev: Event to get details for.
+ * 
+ * Decrease the reference counter of the event detail information in an #event_t. 
+ * If the reference counter is #0 the detail information is freed.
+ */
 void
 event_db_forget_details (event_t ev)
 {
   if (ev->details)
     {
       if (ev->details->ref == 0)
-	fprintf (stderr, "details refcount was already zero!\n");
+        fprintf (stderr, "details refcount was already zero!\n");
       else
-	ev->details->ref--;
+        ev->details->ref--;
 
       if (ev->details->ref == 0)
-	{
-	  event_details_t evd = ev->details;
-	  if (evd->description)
-	    g_free (evd->description);
-	  if (evd->summary)
-	    g_free (evd->summary);
-	  g_slist_free (evd->categories);
-	  g_free (evd);
-	}
+        {
+          event_details_t evd = ev->details;
+          if (evd->description)
+            g_free (evd->description);
+          if (evd->location)
+            g_free (evd->location);
+          if (evd->summary)
+            g_free (evd->summary);
+          g_slist_free (evd->categories);
+          g_free (evd);
+	    }
       ev->details = NULL;
     }
 }
 
+/**
+ * event_db_stop:
+ *
+ * Closes the event database and frees all remaining events in memory.
+ * 
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
 gboolean
 event_db_stop (void)
 {
-  GSList *iter;
+  GList *iter;
 
-  for (iter = one_shot_events; iter; iter = g_slist_next (iter))
-    event_db_destroy(iter->data);
-  g_slist_free(one_shot_events);
+  for (iter = one_shot_events; iter; iter = g_list_next (iter))
+    event_db_destroy (iter->data);
+  g_list_free(one_shot_events);
   one_shot_events = NULL;
 
-  for (iter = recurring_events; iter; iter = g_slist_next (iter))
-    event_db_destroy(iter->data);
-  g_slist_free(recurring_events);
+  for (iter = recurring_events; iter; iter = g_list_next (iter))
+    event_db_destroy (iter->data);
+  g_list_free (recurring_events);
   recurring_events = NULL;
 	
   sqlite_close (sqliteh);
@@ -378,9 +433,9 @@ event_db_stop (void)
 event_t
 event_db_find_by_uid (guint uid)
 {
-  GSList *iter;
+  GList *iter;
     
-  for (iter = one_shot_events; iter; iter = g_slist_next (iter))
+  for (iter = one_shot_events; iter; iter = g_list_next (iter))
     {
       event_t ev = iter->data;
       
@@ -388,7 +443,7 @@ event_db_find_by_uid (guint uid)
 	return ev;
     }
 
-  for (iter = recurring_events; iter; iter = g_slist_next (iter))
+  for (iter = recurring_events; iter; iter = g_list_next (iter))
     {
       event_t ev = iter->data;
        
@@ -402,6 +457,12 @@ event_db_find_by_uid (guint uid)
 	     "warning: event_db_find_by_uid is obsolescent: use the ev->cloned_ev pointer directly.");
 */
 
+/**
+ * event_db_list_destroy:
+ * @l: Event list
+ *
+ * This function frees a given list of events including the list itself. 
+ */
 void
 event_db_list_destroy (GSList *l)
 {
@@ -416,6 +477,15 @@ event_db_list_destroy (GSList *l)
   g_slist_free (l);
 }
 
+/**
+ * event_db_clone:
+ * @ev: Event to clone.
+ * 
+ * Clones a given event. The new event is marked to be a clone instead of 
+ * the original event by setting the #FLAG_CLONE flag.
+ *
+ * Returns: The event clone.
+ */
 event_t
 event_db_clone (event_t ev)
 {
@@ -431,10 +501,12 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 				   gboolean untimed_significant, gboolean alarms, 
 				   guint max)
 {
-  GSList *iter, *iter2;
+  GList *iter;
+  GSList *iter2;
   GSList *list = NULL;
   struct tm tm_event, tm_display;
   long int month_diff;
+  int events = 0;
 	  
   if (end == 0)		/* ??? pb */
     {
@@ -446,7 +518,7 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
       end = mktime (&tm_current);
     }
   
-  for (iter = one_shot_events; iter; iter = g_slist_next (iter))
+  for (iter = one_shot_events; iter; iter = g_list_next (iter))
     {
       event_t ev = iter->data, clone;
       time_t event_start = ev->start;
@@ -476,9 +548,11 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
       clone = event_db_clone(ev);
       clone->start = event_start;
       list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
+      if (++events == max)
+	return list;
     }
   
-  for (iter = recurring_events; iter; iter = g_slist_next (iter))
+  for (iter = recurring_events; iter; iter = g_list_next (iter))
     {
       event_t ev = iter->data, clone;
       time_t event_start = ev->start, clone_start;
@@ -534,6 +608,8 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	  	  else clone->start = clone_start;
 		  clone->flags |= FLAG_RECUR;
 	  	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
+		  if (++events == max)
+		    return list;
 	       }
 	    }
 	  break;
@@ -568,6 +644,8 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	    	  else clone->start = clone_start;
 	    	  clone->flags |= FLAG_RECUR;
 	    	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
+		  if (++events == max)
+		    return list;
 	    	}
 	      }
 	    }
@@ -602,6 +680,8 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	  	  else clone->start = clone_start;
 	  	  clone->flags |= FLAG_RECUR;
 	  	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
+		  if (++events == max)
+		    return list;
 	       }
 	  }
 	  break;
@@ -623,50 +703,55 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	    {
 	      gboolean skip=FALSE;
 	      if (alarms) clone_start+=ev->alarm;
-	      if (r->exceptions) 
-	  	for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
-	  	  if ((long)iter2->data == (long)clone_start) skip=TRUE;
-	       if(alarms && start>clone_start-ev->alarm) skip=TRUE;
-	       if (!skip)
-	       {
-	  	  clone = event_db_clone(ev);
-	  	  if (alarms) clone->start = clone_start-ev->alarm;
-	  	  else clone->start = clone_start;
-	  	  clone->flags |= FLAG_RECUR;
-	  	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
-	       }
-	  }
+	      if (r->exceptions)
+		{
+		  for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
+		    if ((long)iter2->data == (long)clone_start) skip=TRUE;
+		}
+	      if(alarms && start>clone_start-ev->alarm) 
+		skip = TRUE;
+	      if (!skip)
+		{
+		  clone = event_db_clone(ev);
+		  if (alarms) clone->start = clone_start-ev->alarm;
+		  else clone->start = clone_start;
+		  clone->flags |= FLAG_RECUR;
+		  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
+		  if (++events == max)
+		    return list;
+		}
+	    }
 	  break;
   	}
     }
-  
-  if (max && g_slist_length (list) > max)
-    {
-      GSList *return_list = NULL;
-      int n = 0;
-      for (iter = list; iter; iter = g_slist_next (iter))
-	{
-	  event_t ev = iter->data; 
-	  /* Stop if we have enough events */
-	  if (n >= max)
-	    break;
-	  return_list = g_slist_append(return_list, ev);
-	  n++;
-	}
 
-      g_slist_free (list);
-      list = return_list;
-    }
-  
   return list;
 }
 
+/**
+ * event_db_list_for_period:
+ * @start: Start time
+ * @end: End time
+ * 
+ * Create a list of all events for a given period.
+ *
+ * Returns: A list of events.
+ */
 GSList *
 event_db_list_for_period (time_t start, time_t end)
 {
   return event_db_list_for_period_internal (start, end, FALSE, FALSE, FALSE, 0);
 }
 
+/**
+ * event_db_list_alarms_for_period:
+ * @start: Start time
+ * @end: End time
+ * 
+ * Create a list of all alarm events for a given period.
+ *
+ * Returns: A list of events.
+ */
 GSList *
 event_db_list_alarms_for_period (time_t start, time_t end)
 {
@@ -691,12 +776,33 @@ event_db_list_alarms_for_period (time_t start, time_t end)
   return(return_list);
 }
 
+/**
+ * event_db_untimed_list_for_period:
+ * @start: Start time
+ * @end: End time
+ * @yes:
+ * 
+ * Create a list of all untimed events for a given period.
+ *
+ * Returns: A list of events.
+ */
 GSList *
 event_db_untimed_list_for_period (time_t start, time_t end, gboolean yes)
 {
   return event_db_list_for_period_internal (start, end, yes, TRUE, FALSE, 0);
 }
 
+/**
+ * event_db_list_for_future:
+ * @start: Start time
+ * @max: Maximum number of events to retrieve.
+ * 
+ * Get a list of all future events starting at a given time. The total amount 
+ * of events may be limited by the @max parameter. Set this to 0 for an 
+ * unlimited amount of events.
+ *
+ * Returns: A list of events.
+ */
 GSList *
 event_db_list_for_future (time_t start, guint max)
 {
@@ -715,13 +821,14 @@ event_db_list_for_future (time_t start, guint max)
   return_list=event_db_list_for_period_internal (start, shifted_end, FALSE, FALSE, TRUE, 0);
   start=shifted_end;
    
-  for (day_inc=0;day_inc<NUMBER_OF_DAYS;day_inc++) {
-  	return_list = g_slist_concat(return_list, event_db_list_for_period_internal (start,
-	  start+SECONDS_IN_DAY, FALSE, FALSE, FALSE, max));
-	start+=SECONDS_IN_DAY;
-  }
+  for (day_inc=0;day_inc<NUMBER_OF_DAYS;day_inc++) 
+    {
+      return_list = g_slist_concat (return_list, event_db_list_for_period_internal (start,
+										    start+SECONDS_IN_DAY, FALSE, FALSE, FALSE, max));
+      start+=SECONDS_IN_DAY;
+    }
   
-  return(return_list);
+  return return_list;
 }
 
 #define insert_values(db, id, key, format, value)	\
@@ -738,6 +845,7 @@ event_db_write (event_t ev, char **err)
   event_details_t ev_d = event_db_get_details (ev);
   gboolean rc = FALSE;
   GSList *iter;
+  GList **list;
 
   gmtime_r (&ev->start, &tm);
   strftime (buf_start, sizeof (buf_start), 
@@ -746,18 +854,28 @@ event_db_write (event_t ev, char **err)
 
   modified = time (NULL);
 
+  if (!ev->eventid)
+    {
+      ev->eventid = event_db_make_eventid();
+#ifdef DEBUG        
+      printf ("Made new eventid: %s\n", ev->eventid);
+#endif
+    }
+  
   if ((ev_d->summary && insert_values (sqliteh, ev->uid, "summary", "%q", ev_d->summary))
       || (ev_d->description && insert_values (sqliteh, ev->uid, "description", "%q", ev_d->description))
+      || (ev_d->location && insert_values (sqliteh, ev->uid, "location", "%q", ev_d->location))
       || insert_values (sqliteh, ev->uid, "duration", "%d", ev->duration)
       || insert_values (sqliteh, ev->uid, "modified", "%lu", modified)
       || insert_values (sqliteh, ev->uid, "start", "%q", buf_start)
-      || insert_values (sqliteh, ev->uid, "sequence", "%d", ev_d->sequence))
+      || insert_values (sqliteh, ev->uid, "sequence", "%d", ev_d->sequence)
+      || insert_values (sqliteh, ev->uid, "eventid", "%q", ev->eventid))
     goto exit;
 
   for (iter = ev_d->categories; iter; iter = iter->next)
     {
       if (insert_values (sqliteh, ev->uid, "category", "%d", (int)iter->data))
-	goto exit;
+        goto exit;
     }
 
   if (ev->recur)
@@ -768,40 +886,61 @@ event_db_write (event_t ev, char **err)
 	  || insert_values (sqliteh, ev->uid, "rcount", "%d", r->count)
 	  || insert_values (sqliteh, ev->uid, "rincrement", "%d", r->increment)
 	  || insert_values (sqliteh, ev->uid, "rdaymask", "%d", r->daymask))
-	goto exit;
+        goto exit;
 
       if (ev->recur->exceptions)
-      {
-		GSList *iter;
-  		for (iter = ev->recur->exceptions; iter; iter = g_slist_next (iter))
-			if (insert_values (sqliteh, ev->uid, "rexceptions", "%d",
-					(long) iter->data)) goto exit;
-      }
+	{
+	  GSList *iter;
+	  for (iter = ev->recur->exceptions; iter; iter = g_slist_next (iter))
+	    if (insert_values (sqliteh, ev->uid, "rexceptions", "%d",
+			       (long) iter->data)) goto exit;
+	}
       	      
       if (r->end != 0)
-	{
-	  gmtime_r (&r->end, &tm);
-	  strftime (buf_end, sizeof (buf_end), 
-		    (ev->flags & FLAG_UNTIMED) ? "%Y-%m-%d" : "%Y-%m-%d %H:%M",
-		    &tm); 
-	  if (insert_values (sqliteh, ev->uid, "rend", "%q", buf_end)) 
-	    goto exit;
-	}
+        {
+          gmtime_r (&r->end, &tm);
+          strftime (buf_end, sizeof (buf_end), 
+                (ev->flags & FLAG_UNTIMED) ? "%Y-%m-%d" : "%Y-%m-%d %H:%M",
+                &tm); 
+          if (insert_values (sqliteh, ev->uid, "rend", "%q", buf_end)) 
+            goto exit;
+        }
     }
 
   if (ev->flags & FLAG_ALARM)
     {
       if (insert_values (sqliteh, ev->uid, "alarm", "%d", ev->alarm))
-	goto exit;
+       goto exit;
     }
 
   rc = TRUE;
 
- exit:
+  /* update internal list */
+  list = ev->recur ? &recurring_events : &one_shot_events;
+  if (list != ev->list)
+    {
+      *ev->list = g_list_remove (*ev->list, ev);
+      *list = g_list_insert_sorted (*list, ev, (GCompareFunc)event_sort_func);
+      ev->list = list;
+    }
+  else
+    {
+      *ev->list = g_list_sort (*ev->list, (GCompareFunc)event_sort_func);
+    }
+
+exit:
   event_db_forget_details (ev);
   return rc;
 }
 
+/**
+ * event_db_flush:
+ * @ev: Event
+ * 
+ * Remove an event from the database.
+ *
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
 gboolean
 event_db_flush (event_t ev)
 {
@@ -828,11 +967,20 @@ event_db_flush (event_t ev)
   return FALSE;
 }
 
-/* Add an event to both the in-memory list and the SQL database */
+/**
+ * event_db_add:
+ * @ev: Event
+ * 
+ * Add an event to both the in-memory list and the SQL database. This makes 
+ * sure the event is accessible by all application including the application
+ * which adds the event instantly.
+ *
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
 gboolean
 event_db_add (event_t ev)
 {
-  char *err;
+  gchar *err = NULL;
   gboolean rollback = FALSE;
 
   if (sqlite_exec (sqliteh, "begin transaction", NULL, NULL, &err))
@@ -845,10 +993,11 @@ event_db_add (event_t ev)
     goto error;
 
   ev->uid = sqlite_last_insert_rowid (sqliteh);
+  ev->eventid = event_db_make_eventid();
 
   if (event_db_add_internal (ev) == FALSE)
     {
-      err = strdup ("Could not insert event");
+      err = g_strdup ("Could not insert event");
       goto error;
     }
 
@@ -862,12 +1011,18 @@ event_db_add (event_t ev)
   if (rollback)
     sqlite_exec (sqliteh, "rollback transaction", NULL, NULL, NULL);
   gpe_error_box (err);
-  free (err);
+  g_free (err);
   return FALSE;
 }
 
-/* Remove an event from both the in-memory list and the SQL database 
-   from ev pointer */
+/**
+ * event_db_remove:
+ * @ev: Event
+ * 
+ * This function removes an event from the event database. 
+ * 
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
 gboolean
 event_db_remove (event_t ev)
 {
@@ -883,6 +1038,13 @@ event_db_remove (event_t ev)
   return TRUE;
 }
 
+/**
+ * event_db_new:
+ *
+ * Create and initialise a new #event_t event structure.
+ *
+ * Returns: A new event.
+ */
 event_t
 event_db_new (void)
 {
@@ -890,6 +1052,12 @@ event_db_new (void)
   return ev;
 }
 
+/**
+ * event_db_destroy:
+ * @ev: Event
+ * 
+ * Free a given event.
+ */
 void
 event_db_destroy (event_t ev)
 {
@@ -897,10 +1065,21 @@ event_db_destroy (event_t ev)
 
   if (ev->recur)
     event_db__free_recur (ev->recur);
+  if (ev->eventid)
+    g_free (ev->eventid);
 
   event_db__free_event (ev);
 }
 
+/**
+ * event_db_alloc_details:
+ * @ev: Event
+ * 
+ * Allocates a new, clean #event_details_t structure which is attached to
+ * the given event.
+ *
+ * Returns: The new details structure.
+ */
 event_details_t
 event_db_alloc_details (event_t ev)
 {
@@ -909,6 +1088,14 @@ event_db_alloc_details (event_t ev)
   return ev->details;
 }
 
+/**
+ * event_db_get_recurrence:
+ * @ev: Event
+ * 
+ * Retrieves recurrence information for an event.
+ *
+ * Returns: Recurrence information for the given event.
+ */
 recur_t
 event_db_get_recurrence (event_t ev)
 {
@@ -922,6 +1109,29 @@ event_db_get_recurrence (event_t ev)
   return ev->recur;
 }
 
+/**
+ * event_db_clear_recurrence:
+ * @ev: Event
+ * 
+ * Frees the recurrence information of a given event.
+ */
+void
+event_db_clear_recurrence (event_t ev)
+{
+  if (ev->recur)
+    {
+      event_db__free_recur (ev->recur);
+      ev->recur = NULL;
+    }
+}
+
+/**
+ * event_db_refresh:
+ *
+ * Refresh event information from database on disk.
+ *
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
 gboolean
 event_db_refresh (void)
 {
