@@ -32,6 +32,11 @@ typedef enum
   RECUR_YEARLY
 } recur_type_t;
 
+/**
+ * *event_details_t:
+ *
+ * Detail information for an event.
+ */
 typedef struct event_details_s
 {
   guint ref;
@@ -39,8 +44,11 @@ typedef struct event_details_s
   unsigned long sequence;
   time_t modified;
 
-  char *summary;
-  char *description;
+  gchar *summary;
+  gchar *description;
+  gchar *location;  
+  
+  GSList *categories;
 } *event_details_t;
 
 #define FLAG_UNTIMED   (1 << 0)
@@ -57,6 +65,11 @@ struct calendar_time_s
 
 typedef time_t calendar_time_t;
 
+/**
+ * *recur_t:
+ *
+ * Structure to hold recurrence information.
+ */
 typedef struct recur_s
 {
   recur_type_t type;
@@ -69,6 +82,12 @@ typedef struct recur_s
   time_t end;			/* 0 == perpetual */
 } *recur_t;
 
+/**
+ * event_t:
+ *
+ * This data type holds all basic information of an event. More detailed 
+ * event information are covered by the 'details' and 'recur' members.
+ */
 typedef struct event_s
 {
   unsigned long uid;
@@ -83,7 +102,9 @@ typedef struct event_s
   event_details_t details;
   gboolean mark;
   gpointer *cloned_ev;
-  
+  char *eventid;
+
+  GList **list;
 } *event_t;
 
 #define EVENT_DB_USE_MEMCHUNK
@@ -124,26 +145,27 @@ extern gboolean event_db_start (void);
 extern gboolean event_db_refresh (void);
 extern gboolean event_db_stop (void);
 
-extern gboolean event_db_add (event_t);
-extern gboolean event_db_remove (event_t);
+extern gboolean event_db_add (event_t ev);
+extern gboolean event_db_remove (event_t ev);
 
 extern gboolean event_db_flush (event_t ev);
 
-extern event_t event_db_clone (event_t);
+extern event_t event_db_clone (event_t ev);
 extern event_t event_db_new (void);
 extern event_t event_db_find_by_uid (guint uid);
-extern void event_db_destroy (event_t);
+extern void event_db_destroy (event_t ev);
 
-extern event_details_t event_db_alloc_details (event_t);
-extern event_details_t event_db_get_details (event_t);
-extern void event_db_forget_details (event_t);
+extern event_details_t event_db_alloc_details (event_t ev);
+extern event_details_t event_db_get_details (event_t ev);
+extern void event_db_forget_details (event_t ev);
 
 extern GSList *event_db_list_for_period (time_t start, time_t end);
 extern GSList *event_db_list_alarms_for_period (time_t start, time_t end);
 extern GSList *event_db_list_for_future (time_t start, guint max);
 extern GSList *event_db_untimed_list_for_period (time_t start, time_t end, gboolean yes);
-extern void event_db_list_destroy (GSList *);
+extern void event_db_list_destroy (GSList *l);
 
 extern recur_t event_db_get_recurrence (event_t ev);
+extern void event_db_clear_recurrence (event_t ev);
 
 #endif
